"use strict";
/**
 * penguins-eggs-v7
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const pacman_1 = require("../classes/pacman");
const utils_1 = require("./utils");
const exec = require('../lib/utils').exec;
const config_file = '/etc/penguins-eggs.d/eggs.yaml';
const config_tools = '/etc/penguins-eggs.d/tools.yaml';
/**
 * Basket
 */
class Basket {
    constructor() {
        this.lastVersion = '';
        this.lastVersion = '';
    }
    /**
     *
     */
    async last() {
        if (this.lastVersion === '') {
            let arch = 'amd64';
            if (process.arch === 'ia32') {
                arch = 'i386';
            }
            const url = `https://penguins-eggs.net/versions/all/${arch}/`;
            const axios = require('axios').default;
            const res = await axios.get(url);
            const data = res.data;
            // Ordino le versioni
            data.sort((a, b) => (a.version < b.version) ? 1 : ((b.version < a.version) ? -1 : 0));
            this.lastVersion = data[0].version;
        }
        return this.lastVersion;
    }
    /**
     *
     * @param aptVersion
     */
    async get() {
        if (!pacman_1.default.packageIsInstalled('wget')) {
            utils_1.default.titles(`Update from internet`);
            console.log('To download eggs from basket, You need to install wget!\nUse: sudo apt install wget');
            process.exit(1);
        }
        let arch = 'amd64';
        if (process.arch === 'ia32') {
            arch = 'i386';
        }
        else if (process.arch === 'arm') {
            arch = 'armel';
        }
        const url = `https://penguins-eggs.net/versions/all/${arch}/`;
        const axios = require('axios').default;
        const res = await axios.get(url);
        const data = res.data;
        // Ordino le versioni
        data.sort((a, b) => (a.version < b.version) ? 1 : ((b.version < a.version) ? -1 : 0));
        const versions = [];
        for (let i = 0; i < data.length && i <= 3; i++) {
            versions.push(data[i]);
        }
        /**
         * choose the version
         */
        const inquirer = require('inquirer');
        const choices = ['abort'];
        choices.push(new inquirer.Separator('exit without update.'));
        for (let i = 0; i < versions.length; i++) {
            choices.push(versions[i].version);
            choices.push(new inquirer.Separator(versions[i].changelog));
        }
        const questions = [
            {
                type: 'list',
                message: 'select version ',
                name: 'selected',
                choices: choices
            }
        ];
        const answer = await inquirer.prompt(questions);
        if (answer.selected === 'abort') {
            process.exit(0);
        }
        const deb = 'eggs_' + answer.selected + '-1_' + arch + '.deb';
        const download = 'https://sourceforge.net/projects/penguins-eggs/files/packages-deb/' + deb;
        /**
         * downloading
         */
        utils_1.default.titles(`downloading ${deb}`);
        if (await utils_1.default.customConfirm(`Want to download ${deb}`)) {
            process.chdir(`/tmp`);
            if (fs.existsSync(deb)) {
                fs.unlinkSync(deb);
            }
            /**
             * Installing
             */
            await exec(`wget ${download}`);
            utils_1.default.titles(`install ${deb}`);
            if (await utils_1.default.customConfirm(`Want to install ${deb}`)) {
                await exec('apt-get -y purge eggs > /dev/null');
                await exec('rm /usr/lib/penguins-eggs -rf');
                await exec(`dpkg -i ${deb}`);
            }
        }
    }
}
exports.default = Basket;
