"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * penguins-eggs-v8
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const command_1 = require("@oclif/command");
const utils_1 = require("../classes/utils");
const pacman_1 = require("../classes/pacman");
const settings_1 = require("../classes/settings");
const ovary_1 = require("../classes/ovary");
const inquirer = require("inquirer");
const chalk = require("chalk");
const exec = require('../lib/utils').exec;
class Daddy {
    constructor() {
        this.settings = {};
    }
    async helpMe(verbose = false) {
        // Controllo configurazione
        if (!pacman_1.default.configurationCheck()) {
            console.log('- creating configuration dir...');
            await pacman_1.default.configurationInstall(verbose);
        }
        // Controllo prerequisites
        if (!pacman_1.default.prerequisitesCheck()) {
            console.log('- installing prerequisites...');
            pacman_1.default.prerequisitesInstall(verbose);
        }
        if (!pacman_1.default.distroTemplateCheck()) {
            console.log('- distro template install...');
            await pacman_1.default.distroTemplateInstall(verbose);
        }
        // show and edit configuration
        this.settings = new settings_1.default();
        let config = {};
        if (await this.settings.load()) {
            config = this.settings.config;
            // Edito i campi
            let nc = await this.editConfig(config);
            let newConf = JSON.parse(nc);
            // salvo le mdifiche      
            config.snapshot_basename = newConf.snapshot_basename;
            config.snapshot_prefix = newConf.snapshot_prefix;
            config.user_opt = newConf.user_opt;
            config.user_opt_passwd = newConf.user_opt_passwd;
            config.root_passwd = newConf.root_passwd;
            config.theme = newConf.theme;
            if (newConf.compression === 'fast') {
                config.compression = 'lz4';
            }
            else if (newConf.compression === 'normal') {
                config.compression = 'xz';
            }
            else if (newConf.compression === 'max') {
                let filter = 'x86';
                if (process.arch === 'armel') {
                    const filter = 'ARM';
                }
                config.compression = 'xz -Xbcj ' + filter;
            }
            await this.settings.save(config);
            // Controllo se serve il kill
            let flags = '';
            if (verbose) {
                flags = '--verbose ';
            }
            utils_1.default.titles('kill' + flags);
            console.log(chalk.cyan('Daddy, what else did you leave for me?'));
            await this.settings.listFreeSpace();
            if (await utils_1.default.customConfirm()) {
                await exec(`rm ${this.settings.work_dir.path} -rf`);
                await exec(`rm ${this.settings.config.snapshot_dir} -rf`);
            }
            // produce
            if (config.compression === 'lz4') {
                flags += '--fast';
            }
            else if (config.compression === 'xz') {
                flags += '--normal';
            }
            else if (config.compression === 'xz -Xbcj x86') {
                flags += '--max';
            }
            flags += ` --theme=${config.theme} `;
        }
        utils_1.default.titles('produce' + ' ' + command_1.flags);
        console.log(chalk.cyan('Daddy, what else did you leave for me?'));
        const myAddons = {};
        const scriptOnly = false;
        const yolkRenew = false;
        const final = false;
        const ovary = new ovary_1.default(config.snapshot_prefix, config.snapshot_basename, config.theme, config.compression);
        utils_1.default.warning('Produce an egg...');
        if (await ovary.fertilization()) {
            await ovary.produce(scriptOnly, yolkRenew, final, myAddons, verbose);
            ovary.finished(scriptOnly);
        }
    }
    /**
     *
     * @param c
     */
    editConfig(c) {
        console.log(chalk.cyan('Edit and save LiveCD parameters'));
        let compressionOpt = 0;
        if (c.compression === 'xz') {
            compressionOpt = 1;
        }
        else if (c.compression === 'xz -Xbcj x86') {
            compressionOpt = 2;
        }
        if (c.snapshot_prefix === '') {
            c.snapshot_prefix = utils_1.default.snapshotPrefix(this.settings.distro.distroId, this.settings.distro.versionId);
        }
        return new Promise(function (resolve) {
            const questions = [
                {
                    type: 'input',
                    name: 'snapshot_prefix',
                    message: 'LiveCD iso prefix: ',
                    default: c.snapshot_prefix
                },
                {
                    type: 'input',
                    name: 'snapshot_basename',
                    message: 'LiveCD iso basename: ',
                    default: c.snapshot_basename
                },
                {
                    type: 'input',
                    name: 'user_opt',
                    message: 'LiveCD user:',
                    default: c.user_opt
                },
                {
                    type: 'input',
                    name: 'user_opt_passwd',
                    message: 'LiveCD user password: ',
                    default: c.user_opt_passwd
                },
                {
                    type: 'input',
                    name: 'root_passwd',
                    message: 'LiveCD root password: ',
                    default: c.root_passwd
                },
                {
                    type: 'input',
                    name: 'theme',
                    message: 'LiveCD theme: ',
                    default: c.theme
                },
                {
                    type: 'list',
                    name: 'compression',
                    message: 'LiveCD compression: ',
                    choices: ['fast', 'normal', 'max'],
                    default: compressionOpt
                }
            ];
            inquirer.prompt(questions).then(function (options) {
                resolve(JSON.stringify(options));
            });
        });
    }
}
exports.default = Daddy;
