"use strict";
/**
 * penguins-eggs: focal.ts
 *
 * author: Piero Proietti
 * mail: piero.proietti@gmail.com
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Focal = void 0;
const path = require("path");
const fisherman_1 = require("../fisherman");
const exec = require('../../../lib/utils').exec;
/**
 *
 */
class Focal {
    /**
     * @param remix
     * @param distro
     * @param displaymanager
     * @param verbose
     */
    constructor(remix, distro, final, user_opt, verbose = false) {
        this.verbose = false;
        this.final = false;
        this.rootTemplate = './../../../../conf/distros/focal/calamares/';
        this.dirCalamaresModules = '/usr/lib/x86_64-linux-gnu/calamares/modules/';
        this.dirModules = '/etc/calamares/modules/';
        this.fisherman = {};
        this.remix = remix;
        this.distro = distro;
        this.user_opt = user_opt;
        this.verbose = verbose;
        this.final = final;
        if (process.arch === 'ia32') {
            this.dirCalamaresModules = '/usr/lib/calamares/modules/';
        }
        this.rootTemplate = `./../../../../conf/distros/${this.distro.versionLike}/calamares/`;
        this.rootTemplate = path.resolve(__dirname, this.rootTemplate) + '/';
    }
    /**
     *
     */
    async create() {
        const fisherman = new fisherman_1.default(this.distro, this.dirModules, this.dirCalamaresModules, this.rootTemplate, this.verbose);
        await fisherman.settings(this.remix.branding);
        await fisherman.buildModule('partition');
        await fisherman.buildModule('mount');
        await fisherman.moduleUnpackfs();
        await fisherman.buildCalamaresModule('sources-yolk', true);
        await fisherman.buildModule('machineid');
        await fisherman.buildModule('fstab');
        await fisherman.buildModule('locale');
        await fisherman.buildModule('keyboard');
        await fisherman.buildModule('localecfg');
        await fisherman.buildModule('luksbootkeyfile');
        await fisherman.buildModule('users');
        await fisherman.moduleDisplaymanager();
        await fisherman.buildModule('networkcfg');
        await fisherman.buildModule('hwclock');
        await fisherman.contextualprocess('before_bootloader_mkdirs');
        await fisherman.shellprocess('bug-LP#1829805');
        await fisherman.buildModule('initramfs');
        await fisherman.buildModule('grubcfg');
        await fisherman.contextualprocess('before_bootloader');
        await fisherman.buildModule('bootloader');
        await fisherman.contextualprocess('after_bootloader');
        await fisherman.shellprocess('add386arch');
        await fisherman.modulePackages(this.distro, this.final); //
        await fisherman.moduleRemoveuser(this.user_opt);
        await fisherman.buildCalamaresModule('remove-link', true);
        await fisherman.buildCalamaresModule('sources-yolk', false);
        await fisherman.buildCalamaresModule('sources-yolk-unmount', false);
        await fisherman.buildModule('umount');
        await fisherman.buildModule('finished');
    }
}
exports.Focal = Focal;
