"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * penguins-eggs: incubator.ts
 *
 * author: Piero Proietti
 * mail: piero.proietti@gmail.com
 */
const fs = require("fs");
const path = require("path");
const shx = require("shelljs");
const utils_1 = require("../utils");
const buster_1 = require("./distros/buster");
const bullseye_1 = require("./distros/bullseye");
const beowulf_1 = require("./distros/beowulf");
const focal_1 = require("./distros/focal");
const groovy_1 = require("./distros/groovy");
const bionic_1 = require("./distros/bionic");
const exec = require('../../lib/utils').exec;
/**
 *
 */
class Incubator {
    /**
     *
     * @param remix
     * @param distro
     * @param verbose
     */
    constructor(remix, distro, user_opt = 'live', verbose = false) {
        this.verbose = false;
        this.sourcesMedia = false;
        this.sourcesTrusted = true;
        this.remix = remix;
        this.distro = distro;
        this.user_opt = user_opt;
        this.verbose = verbose;
        if (remix.branding === undefined) {
            remix.branding = 'eggs';
        }
    }
    /**
     * config
     */
    async config(final = false) {
        const verbose = true;
        const echo = utils_1.default.setEcho(verbose);
        this.createCalamaresDirs();
        if (this.distro.versionLike === 'buster') {
            const buster = new buster_1.Buster(this.remix, this.distro, final, this.user_opt, this.verbose);
            await buster.create();
        }
        else if (this.distro.versionLike === 'bullseye') {
            const bullseye = new bullseye_1.Bullseye(this.remix, this.distro, final, this.user_opt, this.verbose);
            await bullseye.create();
        }
        else if (this.distro.versionLike === 'beowulf') {
            const beowulf = new beowulf_1.Beowulf(this.remix, this.distro, final, this.user_opt, this.verbose);
            await beowulf.create();
        }
        else if (this.distro.versionLike === 'focal') {
            const focal = new focal_1.Focal(this.remix, this.distro, final, this.user_opt, this.verbose);
            await focal.create();
        }
        else if (this.distro.versionLike === 'groovy') {
            const groovy = new groovy_1.Groovy(this.remix, this.distro, final, this.user_opt, this.verbose);
            await groovy.create();
        }
        else if (this.distro.versionLike === 'bionic') {
            const bionic = new bionic_1.Bionic(this.remix, this.distro, final, this.user_opt, this.verbose);
            await bionic.create();
        }
        this.createBranding();
    }
    /**
     *
     */
    createCalamaresDirs() {
        if (!fs.existsSync('/etc/calamares')) {
            fs.mkdirSync('/etc/calamares');
        }
        if (!fs.existsSync('/etc/calamares/branding')) {
            fs.mkdirSync('/etc/calamares/branding');
        }
        if (!fs.existsSync('/etc/calamares/branding/eggs')) {
            fs.mkdirSync('/etc/calamares/branding/eggs');
        }
        if (!fs.existsSync('/etc/calamares/modules')) {
            fs.mkdirSync('/etc/calamares/modules');
        }
        if (!fs.existsSync('/usr/lib/calamares')) {
            fs.mkdirSync('/usr/lib/calamares/');
        }
        if (!fs.existsSync('/usr/lib/calamares/modules')) {
            fs.mkdirSync('/usr/lib/calamares/modules');
        }
        /**
         * ADDONS
         */
        let calamaresBranding = path.resolve(__dirname, `../../../addons/${this.remix.branding}/theme/branding`);
        if (fs.existsSync(calamaresBranding)) {
            if (!fs.existsSync(`/etc/calamares/branding/${this.remix.branding}`)) {
                fs.mkdirSync(`/etc/calamares/branding/${this.remix.branding}`);
            }
            shx.cp(`${calamaresBranding}/*`, `/etc/calamares/branding/${this.remix.branding}/`);
        }
        else {
            console.log(`${calamaresBranding} not found!`);
            process.exit();
        }
        let calamaresIcon = path.resolve(__dirname, `../../../addons/${this.remix.branding}/theme/artwork/install-debian.png`);
        if (fs.existsSync(calamaresIcon)) {
            shx.cp(calamaresIcon, '/usr/share/icons/');
        }
        else {
            console.log(`${calamaresIcon} not found!`);
            process.exit();
        }
        let calamaresLauncher = path.resolve(__dirname, `../../../addons/${this.remix.branding}/theme/applications/install-debian.desktop`);
        if (fs.existsSync(calamaresLauncher)) {
            shx.cp(calamaresLauncher, '/usr/share/applications/');
        }
        else {
            console.log(`${calamaresLauncher} not found!`);
            process.exit();
        }
        // script di avvio
        shx.cp(path.resolve(__dirname, '../../../assets/calamares/install-debian'), '/sbin/install-debian');
    }
    /**
     *
     */
    createBranding() {
        const branding = require('./branding').branding;
        const dir = `/etc/calamares/branding/${this.remix.branding}/`;
        if (!fs.existsSync(dir)) {
            fs.mkdirSync(dir);
        }
        const file = dir + 'branding.desc';
        const content = branding(this.remix, this.distro, this.verbose);
        write(file, content, this.verbose);
    }
    /**
     * non dovrebbe servire
     */
    async createInstallDebian() {
        const scriptInstallDebian = require('./calamares-modules/scripts/install-debian').installDebian;
        const scriptDir = `/usr/bin/`;
        const scriptFile = scriptDir + 'install-debian';
        const scriptContent = scriptInstallDebian();
        write(scriptFile, scriptContent, this.verbose);
        await exec(`chmod +x ${scriptFile}`);
    }
}
exports.default = Incubator;
/**
 *
 * @param file
 * @param content
 * @param verbose
 */
function write(file, content, verbose = false) {
    if (verbose) {
        console.log(`calamares: create ${file}`);
    }
    fs.writeFileSync(file, content, 'utf8');
}
