import { IMyAddons } from '../interfaces';
import Incubator from './incubation/incubator';
import Settings from './settings';
/**
 * Ovary:
 */
export default class Ovary {
    incubator: Incubator;
    settings: Settings;
    arch_efi: string;
    snapshot_prefix: string;
    snapshot_basename: string;
    theme: string;
    compression: string;
    /**
     * Egg
     * @param compression
     */
    constructor(snapshot_prefix?: string, snapshot_basename?: string, theme?: string, compression?: string);
    /**
     * inizializzazioni che non possono essere messe nel constructor
     * a causa delle chiamate async.
     * @returns {boolean} success
     */
    fertilization(): Promise<boolean>;
    /**
     *
     * @param basename
     */
    produce(scriptOnly: boolean | undefined, yolkRenew: boolean | undefined, final: boolean | undefined, myAddons: IMyAddons, verbose?: boolean): Promise<void>;
    /**
     * Crea la struttura della workdir
     */
    liveCreateStructure(verbose?: boolean): Promise<void>;
    /**
     * editLiveFs
     * - Truncate logs, remove archived log
     * - Allow all fixed drives to be mounted with pmount
     * - Enable or disable password login trhough ssh for users (not root)
     * - Create an empty /etc/fstab
     * - Blanck /etc/machine-id
     * - Add some basic files to /dev
     * - Clear configs from /etc/network/interfaces, wicd and NetworkManager and netman
     */
    editLiveFs(verbose?: boolean): Promise<void>;
    /**
     *  async isoCreateStructure() {
     */
    isolinux(theme?: string, verbose?: boolean): Promise<void>;
    /**
     * copy kernel
     */
    copyKernel(verbose?: boolean): Promise<void>;
    /**
     * squashFs: crea in live filesystem.squashfs
     */
    makeSquashfs(scriptOnly?: boolean, verbose?: boolean): Promise<void>;
    /**
     * info Debian GNU/Linux 10.8.0 "Buster" - Official i386 NETINST 20210206-10:54
     * mkisofs xorriso -as mkisofs -r -checksum_algorithm_iso md5,sha1,sha256,sha512 -V 'Debian 10.8.0 i386 n' -o /srv/cdbuilder.debian.org/dst/deb-cd/out/2busteri386/debian-10.8.0-i386-NETINST-1.iso -jigdo-jigdo /srv/cdbuilder.debian.org/dst/deb-cd/out/2busteri386/debian-10.8.0-i386-NETINST-1.jigdo -jigdo-template /srv/cdbuilder.debian.org/dst/deb-cd/out/2busteri386/debian-10.8.0-i386-NETINST-1.template -jigdo-map Debian=/srv/cdbuilder.debian.org/src/ftp/debian/ -jigdo-exclude boot1 -md5-list /srv/cdbuilder.debian.org/src/deb-cd/tmp/2busteri386/buster/md5-check -jigdo-min-file-size 1024 -jigdo-exclude 'README*' -jigdo-exclude /doc/ -jigdo-exclude /md5sum.txt -jigdo-exclude /.disk/ -jigdo-exclude /pics/ -jigdo-exclude 'Release*' -jigdo-exclude 'Packages*' -jigdo-exclude 'Sources*' -J -joliet-long -cache-inodes -isohybrid-mbr syslinux/usr/lib/ISOLINUX/isohdpfx.bin -b isolinux/isolinux.bin -c isolinux/boot.cat -boot-load-size 4 -boot-info-table -no-emul-boot -eltorito-alt-boot -e boot/grub/efi.img -no-emul-boot -isohybrid-gpt-basdat -isohybrid-apm-hfsplus boot1 CD1
     */
    makeDotDisk(verbose?: boolean): void;
    /**
     * Restituisce true per le direcory da montare con overlay
     *
     * Ci sono tre tipologie:
     *
     * - normal solo la creazione della directory, nessun mount
     * - merged creazione della directory e mount ro
     * - mergedAndOverlay creazione directory, overlay e mount rw
     *
     * @param dir
     */
    mergedAndOvelay(dir: string): boolean;
    /**
     * Ritorna true se c'è bisogno del mount --bind
     *
     * Ci sono tre tipologie:
     *
     * - normal solo la creazione della directory, nessun mount
     * - merged creazione della directory e mount ro
     * - mergedAndOverlay creazione directory, overlay e mount rw
     *
     * @param dir
     *
     * @returns merged
     */
    merged(dir: string): boolean;
    /**
     * Esegue il bind del fs live e
     * crea lo script bind
     *
     * @param verbose
     */
    bindLiveFs(verbose?: boolean): Promise<void>;
    /**
     * ubind del fs live
     * @param verbose
     */
    uBindLiveFs(verbose?: boolean): Promise<void>;
    /**
     * bind dei virtual file system
     */
    bindVfs(verbose?: boolean): Promise<void>;
    /**
     *
     * @param verbose
     */
    ubindVfs(verbose?: boolean): Promise<void>;
    /**
     * create la home per user_opt
     * @param verbose
     */
    createUserLive(verbose?: boolean): Promise<void>;
    /**
     *
     */
    createAutostart(theme: string | undefined, myAddons: IMyAddons, verbose?: boolean): Promise<void>;
    /**
     * Creazione link desktop per lxde
     * @param name
     * @param icon
     */
    private lxdeLink;
    /**
     * Add or remove exclusion
     * @param add {boolean} true = add, false remove
     * @param exclusion {atring} path to add/remove
     */
    addRemoveExclusion(add: boolean, exclusion: string): void;
    /**
     * makeEfi
     * Create /boot and /efi for UEFI
     */
    makeEfi(theme?: string, verbose?: boolean): Promise<void>;
    /**
     * makeIsoImage
     */
    makeIso(scriptOnly?: boolean, verbose?: boolean): Promise<void>;
    /**
     * finished = show the results
     * @param scriptOnly
     */
    finished(scriptOnly?: boolean): void;
}
