"use strict";
/**
 * tools
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const yaml = require("js-yaml");
const settings_1 = require("./settings");
/**
 * Utils: general porpourse utils
 * @remarks all the utilities
  */
class Tools {
    constructor() {
        this.tools_yaml = '/etc/penguins-eggs.d/tools.yaml';
        this.snapshot_dir = '';
        this.snapshot_name = '';
        this.config = {};
    }
    /*
    * Load configuration from /etc/penguins-eggs.yaml
    * @returns {boolean} Success
    */
    async loadSettings() {
        let foundSettings = false;
        if (fs.existsSync(this.tools_yaml)) {
            foundSettings = true;
            const loaded = yaml.load(fs.readFileSync(this.tools_yaml, 'utf-8'));
            this.config.remoteHost = loaded.remoteHost;
            this.config.remoteUser = loaded.remoteUser;
            this.config.remotePathDeb = loaded.remotePathDeb;
            this.config.remotePathDoc = loaded.remotePathDoc;
            this.config.remotePathIso = loaded.remotePathIso;
            this.config.localPathDeb = loaded.localPathDeb;
            this.config.localPathDoc = loaded.localPathDoc;
            this.config.localPathIso = loaded.localPathIso;
            this.config.filterDeb = loaded.filterDeb;
            /**
             * da eggs
             */
            const settings = new settings_1.default();
            settings.load();
            let arch = 'x64';
            if (process.arch === 'ia32') {
                arch = 'i386';
            }
            else if (process.arch === 'armel') {
                arch = 'armel';
            }
            this.snapshot_dir = settings.config.snapshot_dir;
            this.snapshot_name = settings.config.snapshot_prefix + settings.config.snapshot_basename + '-' + arch + '_';
        }
        else {
            console.log(`Can't find: ${this.tools_yaml}`);
            process.exit(1);
        }
        return foundSettings;
    }
}
exports.default = Tools;
