/**
 * penguins-eggs-v7
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
/**
 * Utils: general porpourse utils
 * @remarks all the utilities
 */
export default class Utils {
    /**
     * Restituisce il prefisso della iso
     * @param distroId
     * @param versionId
     */
    static snapshotPrefix(distroId: string, versionId: string): string;
    /**
    * Controlla se il sistema è avviato con systemd
    */
    static isSystemd(): boolean;
    /**
     * ricava path per vmlinuz
     */
    static vmlinuz(): string;
    /**
     * ricava path per initrdImg
     */
    static initrdImg(): string;
    /**
     * Occore vedere un modo per creare machine-id dove non esiste
     */
    static machineId(): string;
    /**
     *
     * @param msg
     */
    static warning(msg?: string): void;
    static error(msg?: string): void;
    /**
     * Return the primary user's name
     */
    static getPrimaryUser(): string;
    /**
     * restituisce uuid
     * @param device
     */
    static uuid(device: string): string;
    /**
     *
     * @param date
     */
    static formatDate(date: Date): string;
    /**
     * return the name of the package: penguins-eggs
     * @returns penguins-eggs
     */
    static getPackageName(): string;
    /**
     * Count the eggs present in the nest
     * @returns {number} Numero degli snapshot presenti
     */
    static getSnapshotCount(snapshot_dir?: string): number;
    /**
     * Get the syze of the snapshot
     * @returns {string} grandezza dello snapshot in Byte
     */
    static getSnapshotSize(snapshot_dir?: string): number;
    /**
     * Return the eggName with architecture and date
     * @param basename
     * @returns eggName
     */
    static getFilename(basename?: string): string;
    /**
     * Calculate the space used on the disk
     * @return {void}
     */
    static getUsedSpace(): number;
    /**
     * Extimate the linuxfs dimension
     * probably is better to rename it as
     * getLiveSpaceRootNeed
     * @returns {number} Byte
     */
    static getLiveRootSpace(type?: string): number;
    /**
     * Return true if i686 architecture
     * @remarks to move in Utils
     * @returns {boolean} true se l'architettura è i686
     * Cambiare con process.arch === 'ia32'
     */
    static isi686(): boolean;
    /**
     * Check se la macchina ha grub adatto ad efi
     */
    static isUefi(): boolean;
    /**
     * Controlla se è un pacchetto deb
     */
    static isDebPackage(): boolean;
    /**
     * Controlla se è un pacchetto sorgente
     */
    static isSources(): boolean;
    /**
     * Controlla se è un pacchetto npm
     */
    static isNpmPackage(): boolean;
    /**
     *
     */
    static rootPenguin(): string;
    /**
     * return the short name of the package: eggs
     * @returns eggs
     */
    static getFriendName(): string;
    /**
     * return the version of the package
     * @returns version example 8.0.0
     */
    static getPackageVersion(): string;
    /**
     * Get author name
     */
    static getAuthorName(): string;
    /**
     * Return the Debian version
     * @remarks to move in Utils
     * @returns {number} Versione di Debian
     */
    static getDebianVersion(): number;
    /**
     * Return true if live system
     * @returns {boolean} isLive
     */
    static isLive(): boolean;
    /**
     * Ritorna vero se path è un mountpoint
     * @param path
     */
    static isMountpoint(path?: string): boolean;
    /**
     * return true if eggs run as root
     * @returns isRoot
     */
    static isRoot(command?: string): boolean;
    /**
     * get the kernel version
     */
    static kernerlVersion(): string;
    /**
     * return the name of network device
     */
    static netDeviceName(): string;
    /**
     * todo
     */
    static netAddress(): string;
    /**
     * todo
     */
    static netMasK(): string;
    /**
     * @returns dns
     */
    static netDns(): string[];
    /**
     * @returns gateway
     */
    static netGateway(): string;
    /**
     * userAdd
     * @param target
     * @param username
     * @param password
     * @param fullName
     */
    static userAdd(target?: string, username?: string, password?: string, fullName?: string): void;
    /**
     * Return an array of the users of the system
     * @remarks to move in Utils
     * @returns {string[]} array di utenti
     */
    static usersList(): string[];
    /**
     * Create folder
     * @param dir
     * @param varius {recursive: true/false}
     */
    static shxMkDir(dir: string | undefined, varius: any): void;
    /**
     * write a file
     * @param file
     * @param text
     */
    static write(file: string, text: string): void;
    /**
     *
     * @param file
     * @param cmd
     */
    static writeX(file: string, cmd: string): void;
    /**
     *
     * @param file
     * @param cmd
     */
    static writeXs(file: string, cmds: string[]): void;
    /**
     *
     * @param msg
     */
    static customConfirm(msg?: string): Promise<boolean>;
    /**
     *
     * @param msg
     */
    static customConfirmCompanion(msg?: string): Promise<any>;
    /**
     *
     * @param msg
     */
    static customConfirmAbort(msg?: string): Promise<any>;
    /**
     * titles
     * Penguin's are gettings alive!
     */
    static titles(command?: string): void;
    /**
     *
     * @param verbose
     */
    static setEcho(verbose?: boolean): object;
}
