"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const shx = require("shelljs");
const fs = require("fs");
const utils_1 = require("./utils");
const execute_1 = require("@getvim/execute");
const pacman_1 = require("./pacman");
const child_process_1 = require("child_process");
const bleach_1 = require("./bleach");
/**
 *
 */
class Yolk {
    constructor() {
        this.dir = '/usr/local/yolk';
    }
    /**
     *
     * @param verbose
     */
    async create(verbose = false) {
        /**
         * riga apt
         *
         * deb [trusted=yes] file:/usr/local/yolk ./
         *
         */
        console.log('updating system...');
        if (!pacman_1.default.commandIsInstalled('dpkg-scanpackages')) {
            process.exit(0);
        }
        child_process_1.execSync('apt-get update --yes');
        if (!this.exists()) {
            shx.exec(`mkdir ${this.dir} -p`);
        }
        else {
            this.clean();
        }
        /**
         * I pacchetti che servono per l'installazione sono solo questi
         */
        const packages = ['grub-pc', 'cryptsetup', 'keyutils'];
        let arch = 'amd64';
        if (process.arch === 'ia32') {
            arch = 'i386';
        }
        else {
            packages.push('grub-efi-amd64');
            packages.push('shim-signed');
        }
        // I Downloads avverranno nell directory corrente
        process.chdir(this.dir);
        // Per tutti i pacchetti cerca le dipendenze, controlla se non siano installate e le scarico.
        for (let i = 0; i < packages.length; i++) {
            let cmd = '';
            console.log(`downloading package ${packages[i]} and it's dependencies...`);
            cmd = `apt-cache depends --recurse --no-recommends --no-suggests --no-conflicts --no-breaks --no-replaces --no-enhances ${packages[i]} | grep "^\\w" | sort -u`;
            let depends = await execute_1.execute(cmd);
            await this.installDeps(depends.split('\n'));
        }
        // Creo Package.gz
        const cmd = 'dpkg-scanpackages -m . | gzip -c > Packages.gz';
        console.log(cmd);
        await execute_1.execute(cmd);
        // Creo Release
        const date = await execute_1.execute('date -R -u');
        const content = `Archive: stable\nComponent: yolk\nOrigin: penguins-eggs\nArchitecture: ${arch}\nDate: ${date}\n`;
        console.log('Creating Release');
        fs.writeFileSync('Release', content);
        console.log('Cleaning apt...');
        const bleach = new bleach_1.default();
        await bleach.clean(verbose);
    }
    /**
     * Svuota la repo yolk
     */
    clean() {
        child_process_1.execSync(`rm ${this.dir}/*`);
    }
    /**
     * Controllo l'esistenza
     */
    exists() {
        const check = `${this.dir}/Packages.gz`;
        return fs.existsSync(check);
    }
    /**
     *
     * @param depends
     */
    async installDeps(depends) {
        // scarico solo le dipendenze non installate
        const toDownloads = [];
        for (let i = 0; i < depends.length; i++) {
            if (!pacman_1.default.packageIsInstalled(depends[i])) {
                toDownloads.push(depends[i]);
            }
        }
        // e li vado a scaricare in /usr/local/yolk
        for (let i = 0; i < toDownloads.length; i++) {
            process.chdir(this.dir);
            const cmd = `apt-get download ${toDownloads[i]}`;
            utils_1.default.warning(`- ${cmd}`);
            await execute_1.execute(cmd);
        }
    }
}
exports.default = Yolk;
