"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const command_1 = require("@oclif/command");
const utils_1 = require("../classes/utils");
const settings_1 = require("../classes/settings");
const incubator_1 = require("../classes/incubation/incubator");
const pacman_1 = require("../classes/pacman");
class Calamares extends command_1.Command {
    constructor() {
        super(...arguments);
        this.remix = {};
        this.incubator = {};
        this.settings = {};
    }
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = this.parse(Calamares);
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        let remove = false;
        if (flags.remove) {
            remove = true;
        }
        let install = false;
        if (flags.install) {
            install = true;
        }
        let final = false;
        if (flags.final) {
            final = true;
        }
        let theme = 'eggs';
        if (flags.theme !== undefined) {
            theme = flags.theme;
        }
        console.log(`theme: ${theme}`);
        if (utils_1.default.isRoot(this.id)) {
            if (await pacman_1.default.isGui()) {
                if (await utils_1.default.customConfirm(`Select yes to continue...`)) {
                    if (install) {
                        utils_1.default.warning('Installing calamares...');
                        await pacman_1.default.calamaresInstall();
                    }
                    if (pacman_1.default.packageIsInstalled('calamares')) {
                        utils_1.default.warning('Configuring calamares...');
                        this.settings = new settings_1.default();
                        if (await this.settings.load()) {
                            await this.settings.loadRemix(this.settings.config.snapshot_basename, theme);
                            this.incubator = new incubator_1.default(this.settings.remix, this.settings.distro, this.settings.config.user_opt, verbose);
                            await this.incubator.config(final);
                        }
                        // force_installer a vero
                        this.settings.config.force_installer = true;
                        this.settings.save(this.settings.config);
                    }
                }
            }
            else {
                console.log(`You cannot use calamares installer without X system!`);
            }
            // Remove calamares
            if (remove && !(install || final)) {
                if (await pacman_1.default.calamaresCheck()) {
                    if (await this.settings.load()) {
                        await pacman_1.default.calamaresRemove();
                        // force_installer a false
                        this.settings.config.force_installer = false;
                        this.settings.save(this.settings.config);
                    }
                }
                else {
                    utils_1.default.warning('Use must use just --remove flag if you decided to remove calamares GUI installer');
                }
            }
        }
    }
}
exports.default = Calamares;
Calamares.description = 'calamares or install or configure it';
Calamares.flags = {
    help: command_1.flags.help({ char: 'h' }),
    verbose: command_1.flags.boolean({ char: 'v' }),
    install: command_1.flags.boolean({ char: 'i', description: 'install calamares and it\'s dependencies' }),
    final: command_1.flags.boolean({ char: 'f', description: 'final: remove calamares and all it\'s dependencies after the installation' }),
    remove: command_1.flags.boolean({ char: 'r', description: 'remove calamares and it\'s dependencies' }),
    theme: command_1.flags.string({ description: 'theme/branding for eggs and calamares' })
};
Calamares.examples = [`~$ sudo eggs calamares \ncreate/renew calamares configuration\'s files\n`, `~$ sudo eggs calamares -i \ninstall calamares and create it\'s configuration\'s files\n`];
