"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * penguins-eggs-v7
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const command_1 = require("@oclif/command");
const utils_1 = require("../classes/utils");
const pacman_1 = require("../classes/pacman");
const bleach_1 = require("../classes/bleach");
const chalk = require("chalk");
const exec = require('../lib/utils').exec;
/**
 *
 */
class Config extends command_1.Command {
    async run() {
        const { flags } = this.parse(Config);
        const nointeractive = flags.nointeractive;
        const verbose = flags.verbose;
        if (!nointeractive) {
            utils_1.default.titles(this.id + ' ' + this.argv);
        }
        if (utils_1.default.isRoot(this.id)) {
            if (flags.clean) {
                await exec('rm /etc/penguins-eggs.d -rf');
            }
            /**
             * Se siamo in un pacchetto npm
             * Aggiunge autocomplete e manPage
             */
            if (!utils_1.default.isNpmPackage()) {
                await pacman_1.default.autocompleteInstall(verbose);
                await pacman_1.default.manPageInstall(verbose);
            }
            // Vede che cosa c'è da fare...
            const i = await Config.thatWeNeed(nointeractive, verbose);
            /**
             * ...e lo fa!
             */
            if (i.needApt || i.configurationInstall || i.configurationRefresh || i.distroTemplate) {
                if (nointeractive) {
                    await Config.install(i, nointeractive, verbose);
                }
                else {
                    if (await utils_1.default.customConfirm()) {
                        await Config.install(i, verbose);
                    }
                }
            }
            else {
                utils_1.default.warning('config: nothing to do!');
            }
        }
    }
    /**
     *
     *
     * @param verbose
     */
    static async thatWeNeed(nointeractive = false, verbose = false) {
        let i = {};
        i.distroTemplate = !pacman_1.default.distroTemplateCheck();
        if (process.arch === 'x64') {
            i.efi = (!utils_1.default.isUefi());
        }
        if (!await pacman_1.default.calamaresCheck() && (await pacman_1.default.isGui())) {
            utils_1.default.warning('config: you are on a graphic system, I suggest to use the GUI installer calamares');
            if (nointeractive) {
                i.calamares = true;
            }
            else {
                i.calamares = (await utils_1.default.customConfirm('Want You install calamares?'));
            }
        }
        i.configurationInstall = !pacman_1.default.configurationCheck();
        if (!i.configurationInstall) {
            i.configurationRefresh = !pacman_1.default.configurationMachineNew();
        }
        i.prerequisites = !await pacman_1.default.prerequisitesCheck();
        if (i.efi || i.calamares || i.prerequisites) {
            i.needApt = true;
        }
        /**
         * Visualizza cosa c'è da fare
         */
        utils_1.default.warning('config: that we need...');
        if (i.needApt && !nointeractive) {
            console.log('- update the system');
            console.log(chalk.yellow('  apt update --yes\n'));
        }
        if (i.efi && !nointeractive) {
            if (process.arch === 'ia32') {
                // do nothing
            }
            else if (process.arch === 'x64') {
                console.log('- install efi packages');
                console.log(chalk.yellow('  apt install -y grub-efi-amd64\n'));
            }
        }
        if (i.prerequisites && !nointeractive) {
            console.log('- install prerequisites');
            console.log(chalk.yellow('  apt install --yes ' + pacman_1.default.debs2line(pacman_1.default.debs4notRemove)));
            const packages = pacman_1.default.packages(verbose);
            console.log(chalk.yellow('  apt install --yes ' + pacman_1.default.debs2line(packages)));
            if (i.configurationInstall) {
                console.log('- creating configuration\'s files...');
                pacman_1.default.configurationInstall(verbose);
            }
            if (i.configurationRefresh) {
                console.log('- refreshing configuration\'s files...');
                pacman_1.default.configurationFresh();
            }
            if (i.distroTemplate) {
                console.log('- copy distro template\n');
            }
            /*
            const packagesLocalisation = Pacman.packagesLocalisation()
            if (packagesLocalisation.length > 0) {
                console.log('- localisation')
                console.log(chalk.yellow('  apt install --yes --no-install-recommends live-task-localisation ' + Pacman.debs2line(packagesLocalisation)) + '\n')
            } else {
                console.log()
            }
            */
        }
        if (i.calamares && !nointeractive) {
            console.log('- install calamares');
            const packages = pacman_1.default.debs4calamares;
            console.log(chalk.yellow('  apt install -y ' + pacman_1.default.debs2line(packages) + '\n'));
        }
        if (i.needApt && !nointeractive) {
            console.log('- cleaning apt\n');
        }
        if (i.configurationInstall) {
            console.log('- creating/updating configuration');
            console.log('  files: ' + chalk.yellow('/etc/penguins-eggs.d/eggs.yaml') + ' and ' + chalk.yellow('/usr/local/share/penguins-eggs/exclude.list\n'));
        }
        else if (i.configurationRefresh) {
            console.log('- refreshing configuration for new machine');
        }
        if (i.needApt && !nointeractive) {
            utils_1.default.warning('Be sure! It\'s just a series of apt install from your repo.\nYou can follows them using flag --verbose');
        }
        return i;
    }
    /**
     *
     * @param i
     * @param verbose
     */
    static async install(i, nointeractive = false, verbose = false) {
        const echo = utils_1.default.setEcho(verbose);
        utils_1.default.warning('config: install');
        if (i.configurationInstall) {
            utils_1.default.warning('creating configuration...');
            await pacman_1.default.configurationInstall(verbose);
        }
        if (i.configurationRefresh) {
            utils_1.default.warning('refreshing configuration for new machine...');
            await pacman_1.default.configurationMachineNew(verbose);
        }
        if (i.distroTemplate) {
            utils_1.default.warning('coping distro templates...');
            await pacman_1.default.distroTemplateInstall(verbose);
        }
        if (i.needApt && !nointeractive) {
            if (!nointeractive) {
                utils_1.default.warning('updating system...');
                await exec('apt-get update --yes', echo);
            }
        }
        if (i.efi) {
            if (nointeractive) {
                utils_1.default.error('config: you are on a system UEFI capable, but I can\'t install grub-efi-amd now!');
                utils_1.default.warning('I suggest You to install grub-efi-amd64 before to produce your ISO.\nJust write:\n    sudo apt install grub-efi-amd64');
            }
            else {
                utils_1.default.warning('Installing uefi support...');
                await exec('apt-get install grub-efi-amd64 --yes', echo); //Pacman.packageInstall('grub-efi-amd64')
            }
        }
        if (i.prerequisites) {
            utils_1.default.warning('Installing prerequisites...');
            await pacman_1.default.prerequisitesInstall(verbose);
        }
        if (i.calamares) {
            if (nointeractive) {
                // solo un avviso
                utils_1.default.error('config: you are on a graphic system, but I can\'t install calamares now!');
                utils_1.default.warning('I suggest You to install calamares GUI installer before to produce your ISO.\nJust write:\n    sudo eggs calamares --install');
            }
            else {
                utils_1.default.warning('Installing calamares...');
                await pacman_1.default.calamaresInstall(verbose);
            }
        }
        if (i.needApt && !nointeractive) {
            utils_1.default.warning('cleaning the system...');
            const bleach = new bleach_1.default();
            await bleach.clean(verbose);
        }
    }
}
exports.default = Config;
Config.description = 'Configure eggs and install packages prerequisites to run it';
Config.aliases = ['prerequisites'];
Config.flags = {
    nointeractive: command_1.flags.boolean({ char: 'n', description: 'assume yes' }),
    clean: command_1.flags.boolean({ char: 'c', description: 'remove old configuration before to create' }),
    help: command_1.flags.help({ char: 'h' }),
    verbose: command_1.flags.boolean({ char: 'v', description: 'verbose' }),
};
Config.examples = [`~$ sudo eggs config\nConfigure eggs and install prerequisites`];
