"use strict";
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const shx = require("shelljs");
const utils_1 = require("../classes/utils");
const settings_1 = require("../classes/settings");
const pacman_1 = require("../classes/pacman");
const chalk = require("chalk");
/**
 * Get informations about system
 */
class Info extends command_1.Command {
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const settings = new settings_1.default();
        settings.load();
        const line = '-----------------------------------------------------------------';
        //console.log(line)
        settings.show();
        const distroId = shx.exec('lsb_release -is', { silent: true }).stdout.trim();
        const versionId = shx.exec('lsb_release -cs', { silent: true }).stdout.trim();
        console.log('distroId:          ' + chalk.cyan(distroId));
        console.log('versionId:         ' + chalk.cyan(versionId));
        console.log('distroLike:        ' + settings.distro.distroLike);
        console.log('versionLike:       ' + settings.distro.versionLike);
        if (await pacman_1.default.prerequisitesCheck()) {
            console.log('eggs prerequisites:' + chalk.bgGreen('ok'));
        }
        else {
            console.log('eggs prerequisites:' + chalk.bgRed('ko'));
        }
        if (await pacman_1.default.configurationCheck()) {
            console.log('configuration:     ' + chalk.bgGreen('ok'));
        }
        else {
            console.log('configuration:     ' + chalk.bgRed('ko'));
        }
        if (await pacman_1.default.isGui()) {
            if (await pacman_1.default.calamaresCheck()) {
                console.log('installer:         ' + chalk.bgGreen('GUI'));
            }
            else {
                console.log('installer:         ' + chalk.bgYellow('CLI') + ' if you want calamares, run ' + chalk.cyan('sudo eggs calamares --install'));
            }
        }
        else {
            console.log('installer:            ' + chalk.bgGreen('CLI'));
        }
        if (process.arch === 'x64') {
            if (!settings.config.make_efi) {
                if (utils_1.default.isUefi()) {
                    console.log('EFI:               ' + chalk.bgRed('ko') + ' run ' + chalk.cyan('sudo eggs dad -c') + ' or edit ' + chalk.cyan('/etc/penguins-eggs.d/eggs.yaml') + ' and set ' + chalk.cyan('make_efi: true'));
                }
                else {
                    console.log('EFI:               ' + chalk.bgRed('ko') + ' run ' + chalk.cyan('apt install grub-efi-amd64') + ' and ' + chalk.cyan('/etc/penguins-eggs.d/eggs.yaml') + ' and set ' + chalk.green('make_efi: true'));
                }
            }
            else {
                console.log('EFI:               ' + chalk.bgGreen('ok'));
            }
        }
        // console.log(line)
        let installType = 'npm package';
        if (utils_1.default.isDebPackage()) {
            installType = 'deb package';
        }
        else if (utils_1.default.isSources()) {
            installType = 'source';
        }
        console.log('eggs is running as:' + chalk.bgGreen(installType));
        if (utils_1.default.isLive()) {
            console.log('system is:         ' + chalk.bgGreen('LIVE') + ' system');
        }
        else {
            console.log('system is:         ' + chalk.bgCyan('INSTALLED'));
        }
        // console.log(line)
    }
}
exports.default = Info;
Info.description = 'informations about system and eggs';
Info.examples = [
    `$ eggs info
You will find here informations about penguin's eggs!
`
];
