"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable unicorn/no-process-exit */
/* eslint-disable no-process-exit */
/* eslint-disable no-console */
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const command_1 = require("@oclif/command");
const utils_1 = require("../classes/utils");
const ovary_1 = require("../classes/ovary");
const config_1 = require("./config");
class Produce extends command_1.Command {
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = this.parse(Produce);
        if (utils_1.default.isRoot(this.id)) {
            /**
             * ADDONS dei vendors
             */
            /*
            let addons = []
            if (flags.addons) {
               console.log(`addons: ${flags.addons}`)
               addons = flags.addons //array
               addons.forEach(addon => {
                  let dirAddon = path.resolve(__dirname, `../../addons/${addon}`)
                  // console.log(`dirAddon: ${dirAddon}`)
                  if (!fs.existsSync(dirAddon)) {
                     console.log(`addon: ${addon} not found`)
                     return
                  }
   
                  let vendorAddon = addon.substring(0, addon.search('/'))
                  // console.log(`vendorAddon: ${vendorAddon}`)
                  let nameAddon = addon.substring(addon.search('/') + 1, addon.length)
                  // Impostazione dei flag theme ed installed_choice se usati come addons
                  if (nameAddon === 'theme') {
                     flags.theme = vendorAddon
                  }
               })
            }
            */
            /**
             * composizione dei flag
             */
            let prefix = '';
            if (flags.prefix !== undefined) {
                prefix = flags.prefix;
            }
            let basename = ''; // se vuoto viene definito da loadsetting (default nome dell'host)
            if (flags.basename !== undefined) {
                basename = flags.basename;
            }
            let compression = ''; // se vuota, compression viene definita da loadsettings, default xz
            if (flags.fast) {
                compression = 'lz4';
            }
            else if (flags.normal) {
                compression = 'xz';
            }
            else if (flags.max) {
                compression = 'xz -Xbcj x86';
            }
            const verbose = flags.verbose;
            const scriptOnly = flags.script;
            const yolkRenew = flags.yolk;
            const final = flags.final;
            let theme = 'eggs';
            if (flags.theme !== undefined) {
                theme = flags.theme;
            }
            const myAddons = {};
            myAddons.adapt = flags.adapt;
            myAddons.rsupport = flags.rsupport;
            myAddons.ichoice = flags.ichoice;
            myAddons.pve = flags.pve;
            const i = await config_1.default.thatWeNeed(verbose);
            if (i.needApt || i.configurationInstall || i.configurationRefresh || i.distroTemplate) {
                if (await utils_1.default.customConfirm(`Select yes to continue...`)) {
                    await config_1.default.install(i, verbose);
                }
            }
            utils_1.default.titles(this.id + ' ' + this.argv);
            const ovary = new ovary_1.default(prefix, basename, theme, compression);
            utils_1.default.warning('Produce an egg...');
            if (await ovary.fertilization()) {
                await ovary.produce(scriptOnly, yolkRenew, final, myAddons, verbose);
                ovary.finished(scriptOnly);
            }
        }
    }
}
exports.default = Produce;
Produce.flags = {
    prefix: command_1.flags.string({ char: 'p', description: 'prefix' }),
    basename: command_1.flags.string({ char: 'b', description: 'basename' }),
    normal: command_1.flags.boolean({ char: 'n', description: 'normal compression' }),
    max: command_1.flags.boolean({ char: 'm', description: 'max compression' }),
    fast: command_1.flags.boolean({ char: 'f', description: 'fast compression' }),
    verbose: command_1.flags.boolean({ char: 'v', description: 'verbose' }),
    yolk: command_1.flags.boolean({ char: 'y', description: '-y force yolk renew' }),
    script: command_1.flags.boolean({ char: 's', description: 'script mode. Generate scripts to manage iso build' }),
    help: command_1.flags.help({ char: 'h' }),
    // addon vendor/addon configurazioni dei vendors
    final: command_1.flags.boolean({ description: 'final: remove eggs prerequisites, calamares and all it\'s dependencies' }),
    theme: command_1.flags.string({ description: 'theme/branding for eggs and calamares' }),
    // addons: flags.string({ multiple: true, description: 'addons to be used' }),
    // addon per prodotti di terze parti, presenti SOLO in eggs
    adapt: command_1.flags.boolean({ description: 'adapt video resolution in VM' }),
    ichoice: command_1.flags.boolean({ description: 'allows the user to choose the installation type cli/gui' }),
    rsupport: command_1.flags.boolean({ description: `remote support via dwagent` }),
    pve: command_1.flags.boolean({ description: `administration of virtual machines (Proxmox-VE)` })
};
Produce.description = 'the system produce an egg: iso image of your system';
Produce.aliases = ['spawn', 'lay'];
Produce.examples = [
    `$ sudo eggs produce \nproduce an ISO called [hostname]-[arch]-YYYY-MM-DD_HHMM.iso, compressed xz (standard compression).\nIf hostname=ugo and arch=i386 ugo-x86-2020-08-25_1215.iso\n`,
    `$ sudo eggs produce -v\nthe same as the previuos, but with more explicative output\n`,
    `$ sudo eggs produce -vf\nthe same as the previuos, compression lz4 (fastest but about 30%\nless compressed than xz)\n`,
    `$ sudo eggs produce -vm\nthe same as the previuos, compression xz (normal compression xz)\n`,
    `$ sudo eggs produce -vm\nthe same as the previuos, compression xz -Xbcj x86 (max compression, about 10%\nmore compressed)\n`,
    `$ sudo eggs produce -vf --basename leo --theme debian --adapt \nproduce an ISO called leo-i386-2020-08-25_1215.iso compression lz4,\nusing Debian theme and link to adapt\n`,
    `$ sudo eggs produce -v --basename leo --theme debian --adapt \nproduce an ISO called leo-i386-2020-08-25_1215.iso compression xz,\nusing Debian theme and link to adapt\n`,
    `$ sudo eggs produce -v --basename leo --rsupport \nproduce an ISO called leo-i386-2020-08-25_1215.iso compression xz, using eggs\ntheme and link to dwagent\n`,
    `$ sudo eggs produce -vs --basename leo --rsupport \nproduce scripts to build an ISO as the previus example. Scripts can be found\nin /home/eggs/ovarium and you can customize all you need\n`
];
