"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const command_1 = require("@oclif/command");
const utils_1 = require("../classes/utils");
const pacman_1 = require("../classes/pacman");
const child_process_1 = require("child_process");
const exec = require('../lib/utils').exec;
/**
 *
 */
class Remove extends command_1.Command {
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = this.parse(Remove);
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        if (utils_1.default.isRoot()) {
            if (utils_1.default.isDebPackage()) {
                /**
                 * debian package
                 */
                if (await utils_1.default.customConfirm()) {
                    if (flags.purge) {
                        child_process_1.execSync('apt-get purge eggs --yes');
                    }
                    else {
                        child_process_1.execSync('apt-get remove eggs --yes');
                    }
                    if (flags.autoremove) {
                        child_process_1.execSync('apt-get autoremove --yes');
                    }
                }
                /**
                 * sources
                 */
            }
            else if (utils_1.default.isSources()) {
                if (await utils_1.default.customConfirm()) {
                    if (flags.autoremove) {
                        await pacman_1.default.prerequisitesRemove();
                    }
                    if (flags.purge) {
                        await pacman_1.default.configurationRemove();
                    }
                    utils_1.default.warning('You are using  eggs as sources. I\'ll NOT remove it');
                }
            }
            else {
                /**
                 * npm package
                 */
                utils_1.default.warning(`You are using eggs as npm package. I'll remove it.`);
                if (await utils_1.default.customConfirm()) {
                    if (flags.autoremove) {
                        await pacman_1.default.prerequisitesRemove();
                    }
                    // Rimuove eggs completion
                    child_process_1.execSync('rm -f /etc/bash_completion.d/eggs.bash');
                    // Rimuove manpages
                    child_process_1.execSync('rm -f /usr/share/man/man1/eggs.1.gz');
                    // purge configurations files
                    if (flags.purge) {
                        await pacman_1.default.configurationRemove();
                    }
                    // Rimuove eggs
                    child_process_1.execSync('npm remove penguins-eggs -g');
                }
            }
        }
    }
}
exports.default = Remove;
Remove.description = 'remove eggs and others stuff';
Remove.examples = [
    `$ sudo eggs remove \nremove eggs\n`,
    `$ sudo eggs remove --prerequisites \nremove eggs, eggs configurations, packages prerequisites\n`,
];
Remove.flags = {
    purge: command_1.flags.boolean({ char: 'p', description: 'remove eggs configurations files' }),
    autoremove: command_1.flags.boolean({ char: 'a', description: 'remove eggs packages dependencies' }),
    help: command_1.flags.help({ char: 'h' }),
    verbose: command_1.flags.boolean({ char: 'v', description: 'verbose' })
};
