"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const command_1 = require("@oclif/command");
const utils_1 = require("../../classes/utils");
const fs = require("fs");
const xdg_1 = require("../../classes/xdg");
class Skel extends command_1.Command {
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = this.parse(Skel);
        let verbose = false;
        if (flags.verbose) {
            verbose = true;
        }
        let user = '';
        if (flags.user) {
            user = flags.user;
        }
        else {
            user = utils_1.default.getPrimaryUser();
        }
        utils_1.default.warning(`user: ${user}`);
        let homeSource = `/home/${user}`;
        if (!fs.existsSync(homeSource)) {
            utils_1.default.error(`User ${user} not exist or not exist a proper home`);
            utils_1.default.warning(`terminate`);
            process.exit(0);
        }
        if (utils_1.default.isRoot()) {
            utils_1.default.titles('skel');
            xdg_1.default.skel(user, verbose);
        }
    }
}
exports.default = Skel;
Skel.description = 'update skel from home configuration';
Skel.aliases = ['skel'];
Skel.examples = [
    `$ eggs skel --user mauro
desktop configuration of user mauro will get used as default`
];
Skel.flags = {
    help: command_1.flags.help({ char: 'h' }),
    user: command_1.flags.string({ char: 'u', description: 'user to be used' }),
    verbose: command_1.flags.boolean({ char: 'v' })
};
