"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * penguins-eggs-v7 based on Debian live
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
const command_1 = require("@oclif/command");
const shx = require("shelljs");
const utils_1 = require("../classes/utils");
const tools_1 = require("../classes/tools");
const pacman_1 = require("../classes/pacman");
const basket_1 = require("../classes/basket");
const exec = require('../lib/utils').exec;
/**
 *
 */
class Update extends command_1.Command {
    async run() {
        utils_1.default.titles(this.id + ' ' + this.argv);
        const { flags } = this.parse(Update);
        if (utils_1.default.isRoot(this.id)) {
            utils_1.default.titles(this.id + ' ' + this.argv);
            if (utils_1.default.isSources()) {
                utils_1.default.warning(`You are on penguins-eggs v. ${utils_1.default.getPackageVersion()} from sources`);
            }
            else if (utils_1.default.isDebPackage()) {
                utils_1.default.warning(`You are on eggs-${utils_1.default.getPackageVersion()} installed as package .deb`);
            }
            else {
                utils_1.default.warning(`You are on penguins-eggs@${utils_1.default.getPackageVersion()} installed as package npm`);
            }
            let apt = false;
            let aptVersion = '';
            if (await pacman_1.default.packageAptAvailable('eggs')) {
                apt = true;
                aptVersion = await pacman_1.default.packageAptLast('eggs');
                utils_1.default.warning('eggs-' + aptVersion + ' is available via apt');
            }
            else {
                utils_1.default.warning('eggs is not available your repositories');
            }
            let npmVersion = await pacman_1.default.packageNpmLast('penguins-eggs');
            if (npmVersion !== '') {
                utils_1.default.warning('penguins-eggs@' + npmVersion + ' available via npm');
            }
            const basket = new basket_1.default();
            const basketVersion = await basket.last();
            if (basketVersion !== '') {
                utils_1.default.warning('eggs-' + basketVersion + '-1.deb available in basket');
            }
            /**
             * Se è specificato il metodo di aggiornamento
             * e, questo corrisponde al tipo di pacchetto
             * installato
             */
            if (flags.npm || flags.apt || flags.basket) {
                if (utils_1.default.isDebPackage() && flags.apt) {
                    await this.getDebFromApt();
                }
                else if (utils_1.default.isDebPackage() && flags.basket) {
                    await basket.get();
                }
                else if (flags.npm) {
                    await this.getFromNpm();
                }
                else {
                    await this.chooseUpdate();
                }
            }
            else {
                await this.chooseUpdate();
            }
        }
    }
    /**
     * Altrimenti, seleziona il tipo di
     * aggiornamento desiderato
     * indipendentemente dal flag
     */
    async chooseUpdate() {
        let basket = new basket_1.default();
        console.log();
        const choose = await this.chosenDeb();
        utils_1.default.titles(`updating via ${choose}`);
        if (choose === 'apt') {
            await this.getDebFromApt();
        }
        else if (choose === 'basket') {
            await basket.get();
        }
        else if (choose === 'npm') {
            await this.getFromNpm();
        }
        else if (choose === 'lan') {
            await this.getDebFromLan();
        }
        else if (choose === 'manual') {
            this.getDebFromManual();
        }
        else if (choose === 'sources') {
            this.getFromSources();
        }
    }
    /**
     *
     */
    async chosenDeb() {
        const inquirer = require('inquirer');
        const choices = ['abort'];
        choices.push(new inquirer.Separator('exit from update'));
        choices.push('basket');
        choices.push(new inquirer.Separator('select, download and update from basket'));
        choices.push('lan');
        choices.push(new inquirer.Separator('automatic import and update from lan'));
        choices.push('npm');
        choices.push(new inquirer.Separator('automatic import and update from npmjs.com'));
        choices.push('manual');
        choices.push(new inquirer.Separator('manual download from sourceforge.net and update with dpkg'));
        choices.push('sources');
        choices.push(new inquirer.Separator('download sources from github.com'));
        const questions = [
            {
                type: 'list',
                message: 'select update method',
                name: 'selected',
                choices: choices
            }
        ];
        const answer = await inquirer.prompt(questions);
        if (answer.selected === 'abort') {
            process.exit(0);
        }
        return answer.selected;
    }
    /**
     * getFromNpm
     */
    async getFromNpm() {
        shx.exec(`npm update ${utils_1.default.getPackageName()}@latest -g`);
    }
    /**
     *
     * @param aptVersion
     */
    getFromSources() {
        console.log('You can upgrade getting a new version from git:');
        console.log('cd ~/penguins-eggs');
        console.log('git pull');
        console.log('');
        console.log('Or You can also create a fresh installation on your hone:');
        console.log('cd ~');
        console.log('git clone https://github.com/pieroproietti/penguins-eggs');
        console.log('');
        console.log('Before to use eggs, remember to install npm packages:');
        console.log('cd ~/penguins-eggs');
        console.log('npm install');
    }
    /**
     *
     */
    async getDebFromManual() {
        console.log('Download package from: \n\nhttps://sourceforge.net/projects/penguins-eggs/files/packages-deb/');
        console.log('\nand install it with:');
        const basket = new basket_1.default();
        console.log('\nsudo dpkg -i eggs_' + await basket.last() + '-1.deb');
    }
    /**
     * download da LAN
     */
    async getDebFromLan() {
        const Tu = new tools_1.default;
        await Tu.loadSettings();
        let arch = 'amd64';
        if (process.arch === 'ia32') {
            arch = 'i386';
        }
        utils_1.default.warning(`import from lan`);
        console.log();
        let cmd = `scp ${Tu.config.remoteUser}@${Tu.config.remoteHost}:${Tu.config.remotePathDeb}${Tu.config.filterDeb}${arch}.deb /tmp`;
        await exec(cmd, { echo: true, capture: true });
        console.log('sudo dpkg -i /tmp/eggs_*.deb');
    }
    /**
     *
     */
    async getDebFromApt() {
        if (pacman_1.default.packageAptAvailable('eggs')) {
            await exec(`apt reinstall eggs`);
        }
        else {
            console.log(`eggs is not present in your repositories`);
            console.log(`but you can upgrade from internet`);
        }
    }
}
exports.default = Update;
Update.description = "update the penguin's eggs tool";
Update.examples = [`$ eggs update\nupdate/upgrade the penguin's eggs tool`];
Update.flags = {
    help: command_1.flags.help({ char: 'h' }),
    apt: command_1.flags.boolean({ char: 'a', description: 'if eggs package is .deb, update from distro repositories' }),
    basket: command_1.flags.boolean({ char: 'b', description: 'if eggs package is .deb, update from eggs basket' }),
    npm: command_1.flags.boolean({ char: 'n', description: 'if eggs package is .npm, update from npmjs.com' }),
    verbose: command_1.flags.boolean({ char: 'v', description: 'verbose' })
};
