"use strict";
/**
 * Executes shell command as it would happen in BASH script
 * @param {string} command
 * @param {Object} [options] Object with options.
 *                           Set `echo` to TRUE, to echo command passed.
 *                           Set `ignore` to TRUE to ignore stdout
 *                           Set `capture` to TRUE, to capture and return stdout.
 *
 * @returns {Promise<{code: number, data: string | undefined, error: Object}>}
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.exec = void 0;
function exec(command, { echo = false, ignore = false, capture = false } = {}) {
    if (echo) {
        console.log(command);
    }
    const spawn = require('child_process').spawn;
    // const childProcess = spawn('bash', ['-c', command], { stdio: capture ? 'pipe' : 'inherit' });
    // const childProcess = spawn('bash', ['-c', command], { stdio: capture ? 'pipe' : ignore ? 'ignore' : 'inherit' });
    const childProcess = spawn('bash', ['-c', command], {
        stdio: ignore ? 'ignore' : capture ? 'pipe' : 'inherit'
    });
    return new Promise((resolve, reject) => {
        let stdout = '';
        if (capture) {
            childProcess.stdout.on('data', (data) => {
                stdout += data;
            });
        }
        childProcess.on('error', function (error) {
            reject({ code: 1, error: error });
        });
        childProcess.on('close', function (code) {
            if (code > 0) {
                reject({ code: code, error: 'Command failed with code ' + code });
            }
            else {
                resolve({ code: code, data: stdout });
            }
        });
    });
}
exports.exec = exec;
